/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.auth;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.opengroup.osdu.core.common.model.entitlements.AuthorizationResponse;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.provider.interfaces.IAuthorizationService;
import org.opengroup.osdu.notification.di.RequestInfoExt;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubRequestBodyExtractor;
import org.opengroup.osdu.notification.provider.interfaces.IServiceAccountValidator;
import org.opengroup.osdu.notification.utils.Config;
import org.powermock.modules.junit4.PowerMockRunner;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(PowerMockRunner.class)
public class AuthorizationFilterTest {

    private static final String ROLE1 = "role1";
    private static final String ROLE2 = "role2";
    private static final String ROLE3 = "cron.job";
    private static final String ROLE4 = "notification.pubsub";
    private static final String TEST_PUBSUB_JWT = "Bearer test.jwt.token";

    @Mock
    private DpsHeaders headers;

    @Mock
    private HttpServletRequest request;

    @Mock
    private RequestInfoExt requestInfo;

    @Mock
    private IAuthorizationService authorizationService;

    @Mock
    private IServiceAccountValidator validator;

    @Mock
    private IPubsubRequestBodyExtractor extractor;

    @InjectMocks
    private AuthorizationFilter sut;

    @Before
    public void setup() {
        when(headers.getAuthorization()).thenReturn("Bearer 123456");
        when(headers.getPartitionId()).thenReturn("tenant1");
        when(requestInfo.getHeaders()).thenReturn(headers);
    }

    @Test
    public void should_authenticateRequest_when_resourceIsRolesAllowedAnnotated() {
        final String USER_EMAIL = "test@test.com";
        AuthorizationResponse authorizationResponse = AuthorizationResponse.builder().user(USER_EMAIL).build();
        when(this.authorizationService.authorizeAny(any(), eq(ROLE1), eq(ROLE2))).thenReturn(authorizationResponse);

        assertTrue(this.sut.hasAnyPermission(ROLE1, ROLE2));
        verify(headers).put(DpsHeaders.USER_EMAIL, "test@test.com");
    }

    @Test(expected = AppException.class)
    public void should_throwAppError_when_noAuthzProvided() {
        when(headers.getAuthorization()).thenReturn("");
        final String USER_EMAIL = "test@test.com";
        this.sut.hasAnyPermission(ROLE1, ROLE2);
        assertEquals(USER_EMAIL, this.headers.getUserEmail());
    }

    @Test
    public void should_authenticateRequest_when_isCronJob() {
        when(this.requestInfo.isCronRequest()).thenReturn(true);

        assertTrue(this.sut.hasAnyPermission(ROLE3));
        verify(headers).put(DpsHeaders.USER_EMAIL, Config.CRON);
    }

    @Test(expected = AppException.class)
    public void should_notAuthenticateRequest_when_appEngineCronHeaderIsNotAsExpectedForCronJob() {
        when(this.authorizationService.authorizeAny(any(), any())).thenThrow(new AppException(403, "", ""));
        when(this.requestInfo.isCronRequest()).thenReturn(false);
        this.sut.hasAnyPermission(ROLE3);
    }

    @Test
    public void should_authenticateRequest_when_isFromPubsubAndUserIdentityIsCorrect() {
        when(headers.getAuthorization()).thenReturn(TEST_PUBSUB_JWT);
        when(this.validator.isValidServiceAccount(any(), any())).thenReturn(true);
        when(this.authorizationService.authorizeAny(any(), eq(ROLE4))).thenReturn(AuthorizationResponse.builder().user("test@test.com").build());
        this.sut.hasAnyPermission(ROLE4);
    }

    @Test(expected = AppException.class)
    public void should_notAuthenticateRequest_when_isFromPubsubAndUserIdentityIsNotOps() {
        when(headers.getAuthorization()).thenReturn(TEST_PUBSUB_JWT);
        when(this.validator.isValidServiceAccount(any(), any())).thenReturn(false);
        when(this.authorizationService.authorizeAny(any(), any())).thenThrow(new AppException(403, "", ""));
        this.sut.hasAnyPermission(ROLE4);
    }

    @Test
    public void should_notAuthenticateRequest_when_isFromPubsubAndUserIdentityIsOps() {
        Map<String, String> attributes = new HashMap<>();
        attributes.put(DpsHeaders.DATA_PARTITION_ID, "tenant1");
        final String USER_EMAIL = "test@test.com";
        AuthorizationResponse authorizationResponse = AuthorizationResponse.builder().user(USER_EMAIL).build();

        when(this.extractor.extractAttributesFromRequestBody()).thenReturn(attributes);
        when(headers.getAuthorization()).thenReturn(TEST_PUBSUB_JWT);
        when(this.validator.isValidServiceAccount(any(), any())).thenReturn(false);
        when(this.authorizationService.authorizeAny(any(), eq(ROLE4), eq(ROLE2))).thenReturn(authorizationResponse);
        this.sut.hasAnyPermission(ROLE4, ROLE2);
        verify(headers).put(DpsHeaders.USER_EMAIL, USER_EMAIL);
    }
}
